use std::collections::HashSet;
use crypto::digest::Digest;
use crypto::sha2::Sha256;

#[derive(Debug)]
pub struct Block {
    pub height: u32,
    pub hash: String,
    pub previous_block_hash: String,
    pub data: HashSet<u8>,
    pub nonce: u32
}

const DIFFICULTY: u32 = 3;

impl Block {

    pub fn mine_block(&mut self) {
        loop {
            self.nonce += 1;
            let hash = self.calculate_block_hash();
            let prefix = "0".repeat(DIFFICULTY as usize);
            if hash.starts_with(prefix.as_str()) {
                self.hash = hash;
                break;
            }
        }
    }

    pub fn calculate_block_hash(&self) -> String {
        // TODO: use byte array instead of string for hashing
        let data = format!("{}, {}, {:?}, {}", self.height, self.previous_block_hash, self.data, self.nonce);
        let mut hasher = Sha256::new();
        hasher.input(data.as_bytes());
        hasher.result_str()
    }
}